// Aide Forth 2000 core

create ;
  ]
  ' return ,
  RunTime       // [ ' [ , ]
  return [
immediate

create :
  ]
  create
  CompileTime   // [ ' ] , ]
;

: (
  " )" ParseWord
  drop
; immediate

: (<build)
  create
  ' (does>) ,
;

: <build
  :
  ' (<build) ,
  ' (ilit) ,
  here # 0 ,
  ' , ,
;

: does>
  here # 8 + swap !
  ' return ,
; immediate

: if
  ' ?branch ,
  here # -1 ,
  # 100
; immediate

: else
  # 100 = assert
  ' branch ,
  here # -1 ,
  swap // here oldhere
  here swap !
  # 100
; immediate

: endif
  # 100 = assert
  here swap !
; immediate

: then
  # 100 = assert
  here swap !
; immediate

: while
  here
  # 101
; immediate

: loop
  # 101 = assert
  ' ?branch ,
  here # -1 ,
  # 102
; immediate

: endloop
  # 102 = assert
  ' branch ,
  swap ,
  here swap !
; immediate

: true
  # -1
;

: false
  # 0
;

: dciInsertText
  dup View . InsertText ( .) quit
;

// Aide Custom words

: CalcMargin
  View .@ CursorStr Trim " " =  if                     // if Trim(View.CursorStr) = ''
    View .@ CursorCol # 1 - View .! Margin             //   then View.Margin := View.CursorCol - 1
  else
    View .@ CursorStr Length                           //   else View.Margin := Length(View.CursorStr) - Length(TrimLeft(View.CursorStr));
    View .@ CursorStr TrimLeft Length - View .! Margin
    View .@ Margin View .@ CursorCol > if
      View .@ CursorCol # 1 - View .! Margin
    endif
  endif
;

// This is an simple BeginEnd overwritten on Forth. Use the Source, Luke
// Word that types heuristic: "begin end;" or "begin end" before "else"
// or "begin end {for};" after "for" and so on
// This is an example of cool forth aide programming

: BeginEnd
  CalcMargin
  " begin" View . InsertText ( .)                      //  View.InsertText('begin');
  View . mCRLF ()                                      //  View.mCRLF;
  "   " View . InsertText ( .)                         //  View.InsertText('  ');
  View .@ CursorCol                                    //  SaveCursorCol := View.CursorCol;
  View .@ CursorLine                                   //  SaveCursorLine := View.CursorLine;
  View .@ CursorStr Length # 1 + View .! CursorCol     //  View.CursorCol := Length(View.CursorStr + 1);
  View . mCRLF ()                                      //  View.mCRLF;
  " end" View . InsertText ( .)                        //  View.InsertText('end;');
    // ; not allowed before else, external feature
  View .@ CursorCol View .@ CursorLine
  Parser . SkipBlanks () Parser .@ IdentOnCursor " else" =
  -rot View .! CursorLine View .! CursorCol if
  else
    " ;" View . InsertText ( .)
  endif
  View .! CursorLine                                   //  View.CursorCol := SaveCursorCol;
  View .! CursorCol                                    //  View.CursorLine := SaveCursorLine;
;

: DeclareVariable ( VarName -- )
  // Lets suppose that i,j and k is integer
  dup " i" =
  over " j" = |
  over " k" = |
  over " l" = | if
    " integer" DeclareVar drop return
  endif
  // r is TRect
  dup " r" = if
    " TRect" DeclareVar drop return
  endif
  // p is TPoint
  dup " p" = if
    " TPoint" DeclareVar drop return
  endif
  // E is Exception
  dup " e" = if
    " Exception" DeclareVar drop return
  endif
  // s and z is string
  dup " s" =
  over " z" = | if
    " string" DeclareVar drop return
  endif
  dup " u" = if
    " IUnknown" DeclareVar drop return
  endif
  dup " a" = if
    " any" DeclareVar drop return
  endif
  // ..ID is TID
  dup " ID" swap IsSuffix if
    " TID"
  else
    " "
  endif
  DeclareVar drop
;

// Word that declare variable on cursor or converts quotation to resourcestring
// Based on using internal aide features DeclareResourceString and DeclareVar
: DeclareVarOnCursor
  Parser .@ QuoteOnCursor dup " " = if
    drop
  else
    dup Length View .! DeleteText
    DeclareResourceString
    View . InsertText ( .)
    return
  endif

  Parser .@ IdentOnCursor
  DeclareVariable
;

: CalcVarSpecification ( -- VarType VarName )
  View .@ CursorCol                                    //  SaveCursorCol := View.CursorCol;
  View .@ CursorLine                                   //  SaveCursorLine := View.CursorLine;
  dup # 1 - View .! CursorLine # 1 View .! CursorCol
  Parser . SkipBlanks () Parser .@ IdentOnCursor          ( CursorCol CursorLine Variable -- )
  -rot
  Parser . SkipToken ()
  Parser . SkipBlanks () View .@ CursorChar " :" =         // Look for variable's type <var> := <type>.Create;
  View . FeedChar () View .@ CursorChar " =" = & if
    View . FeedChar ()
    Parser . SkipBlanks () Parser .@ IdentOnCursor
    " T" over IsPrefix if
    else
      drop " "
    endif
  else
    " *"
  endif  ( VarName CursorCol CursorLine VarType -- )
  -rot
  View .! CursorLine                                   //  View.CursorCol := SaveCursorCol;
  View .! CursorCol                                    //  View.CursorLine := SaveCursorLine;
  swap                                                 ( VarType VarName -- )
;

// Heuristicaly inserts try finally block with
//   <variable>.Free and tries to declare variable.
// Variable's specification is maintaineds from upper line of code

: TryFinally
  CalcVarSpecification
  CalcMargin
  " try" View . InsertText ( .)                        //  View.InsertText('try');
  View . mCRLF ()                                      //  View.mCRLF;
  "   " View . InsertText ( .)                         //  View.InsertText('  ');
  View .@ CursorCol                                    //  SaveCursorCol := View.CursorCol;
  View .@ CursorLine                                   //  SaveCursorLine := View.CursorLine;
  View .@ CursorStr Length # 1 + View .! CursorCol     //  View.CursorCol := Length(View.CursorStr + 1);
  rot
  View . mCRLF ()                                      //  View.mCRLF
  " finally" View . InsertText ( .)                    //  View.InsertText('finally');
  View . mCRLF ()                                      //  View.mCRLF
  "   " View . InsertText ( .)                         //  View.InsertText('  ');
  dup View . InsertText ( .) -rot
  " .Free;" View . InsertText ( .)                     //  View.InserText('.Free');
  View . mCRLF ()                                      //  View.mCRLF
  " end;" View . InsertText ( .)                       //  View.InsertText('end;');
  View .! CursorLine                                   //  View.CursorCol := SaveCursorCol;
  View .! CursorCol                                    //  View.CursorLine := SaveCursorLine;
  over " *" = if
    drop return
  endif
  over " " = if
    DeclareVariable
    drop
  else
    swap DeclareVar
  endif
;

: FormTryFinally
  CalcVarSpecification
  CalcMargin
  " try" View . InsertText ( .)                        //  View.InsertText('try');
  View . mCRLF ()                                      //  View.mCRLF;
  "   if " View . InsertText ( .)
  dup View . InsertText ( .)
  " .ShowModal = mrOk then" View . InsertText ( .)
  View . mCRLF ()
  "     begin" View . InsertText ( .)
  View . mCRLF ()
  "       " View . InsertText ( .)                     //  View.InsertText('      ');
  View .@ CursorCol                                    //  SaveCursorCol := View.CursorCol;
  View .@ CursorLine                                   //  SaveCursorLine := View.CursorLine;
  View .@ CursorStr Length # 1 + View .! CursorCol     //  View.CursorCol := Length(View.CursorStr + 1);
  rot
  View . mCRLF ()                                      //  View.mCRLF
  "     end;" View . InsertText ( .)
  View . mCRLF ()
  " finally" View . InsertText ( .)                    //  View.InsertText('finally');
  View . mCRLF ()                                      //  View.mCRLF
  "   " View . InsertText ( .)                         //  View.InsertText('  ');
  dup View . InsertText ( .) -rot
  " .Release;" View . InsertText ( .)                  //  View.InserText('.Free');
  View . mCRLF ()                                      //  View.mCRLF
  " end;" View . InsertText ( .)                       //  View.InsertText('end;');
  View .! CursorLine                                   //  View.CursorCol := SaveCursorCol;
  View .! CursorCol                                    //  View.CursorLine := SaveCursorLine;
  over " *" = if
    drop return
  endif
  over " " = if
    DeclareVariable
    drop
  else
    swap DeclareVar
  endif
;

// This word is called every time AideSS wants to insert SourceSafe Keywords
// for easy file locating in SourceSafe's database
: InsertSourceSafeKeyword
  View .@ CursorCol
  View .@ CursorLine
  # 0 View .! CursorLine
  # 1 View .! CursorCol
    // $NoKeywords: Archive Author Date Revision Workfile $
  " // $Archive: $ $Author: $ $Date: $ $Revision: $ $Workfile: $" View . InsertText ( .)
  View . CRLF ()
  View . CRLF ()
  # 2 + View .! CursorLine
  View .! CursorCol
;

: JumpMethodDialog
  " hiJumpMethod" " Jump method" " Enter method name :" Parser .@ IdentOnCursor InputQueryWithHistory
  if
    JumpMethod
  else
    drop
  endif
;

: JumpClassDialog
  " hiJumpClass" " Jump class" " Enter class name:" Parser .@ IdentOnCursor InputQueryWithHistory
  if
    JumpClass
  else
    drop
  endif
;

: JumpMethodOnCursor
  Parser .@ IdentOnCursor
  dup " " = if
    drop
  else
    JumpMethod
  endif
;

: JumpClassOnCursor
  Parser .@ IdentOnCursor
  dup " " = if                                         //  ClasName := Source.GetIdentOnCursor;
    drop                                               //  if ClasName <> ''
  else                                                 //    then _JumpClass(Source, ClasName);
    " P" over IsPrefix if   // if PRecordInfo then jump to TRecordInfo
      # 2 over Length Copy
      " T" swap Concat
    endif
    JumpClass
  endif
;

: ForList
  " List for" " Enter variable name:" " i" InputQuery if
    dup " integer" DeclareVar drop
  else
    drop return
  endif
  View .@ CursorCol # 1 - View .! Margin
  " for " View . InsertText ( .)
  View . InsertText ( .) "  := 0 to " View . InsertText ( .)
  View .@ CursorCol View .@ CursorLine
  " .Count - 1 do" View . InsertText ( .) View . mCRLF ()
  "   begin" View . InsertText ( .) View . mCRLF ()
  "     ;" View . InsertText ( .) View . mCRLF ()
  "   end;"  View . InsertText ( .) View . CRLF ()
  View .! CursorLine View .! CursorCol
;

: TryFinallyEx
  " try finally" ExecCodeTemplate
;

: GeneralList
  " TGL" ExecCodeTemplate
;
: StructuredTreeList
  " TSTL" ExecCodeTemplate
;

: JumpInitialization
  Source .@ ImplEndLine View .! CursorLine # 1 View .! CursorCol // Jump ImplEndLine

  Parser .@ IdentOnCursor " initialization" = if
    Source .@ ImplEndLine # 1 + View .! CursorLine # 1 View .! CursorCol // Next line
    Parser . SkipBlanks () Parser .@ IdentOnCursor " begin" = if // begin found here
      Source .@ ImplEndLine # 2 + View .! CursorLine // moving to the next line
    endif
    # 1 View .! CursorCol
    Parser . SkipBlanks ()
  else                         // Inserting initialization part
    " initialization" View . InsertText ( .) View . CRLF ()
    "   begin" View . InsertText ( .) View . CRLF ()
    "     "    View . InsertText ( .)
    View .@ CursorCol View .@ CursorLine
    View . CRLF ()
    "   end;"  View . InsertText ( .) View . CRLF ()
    View . CRLF ()
    View .! CursorLine View .! CursorCol
  endif
;

: RegisterInterface
  Parser .@ IdentOnCursor dup " " = if
    drop
    # 1 View .! CursorCol
    Parser . SkipBlanks ()
    Parser .@ IdentOnCursor // Getted identifier
  endif
  JumpInitialization
  View .@ CursorCol # 1 - View .! Margin
  " RegisterGUIDName(" View . InsertText ( .)
    dup View . InsertText ( .)
    " , '" View . InsertText ( .)
    View . InsertText ( .)
    " ');" View . InsertText ( .)
  View . mCRLF ()
;

: Startup
//  " AIDEMenu" # -1 " -" " " " " AddMenuItem drop
  " AIDEMenu" # -1 " &Begin End" " BeginEnd" " Alt+B" AddMenuItem drop
  " AIDEMenu" # -1 " &Try Finally" " TryFinally" " Alt+Y" AddMenuItem drop
  " AIDEMenu" # -1 " &Form Try Finally" " FormTryFinally" " Ctrl+Alt+F" AddMenuItem drop
  " AIDEMenu" # -1 " For &List" " ForList" " Alt+L" AddMenuItem drop
  " AIDEMenu" # -1 " Declare &Var on Cursor" " DeclareVarOnCursor" " Alt+Space" AddMenuItem drop
  " AIDEMenu" # -1 " &Forth Code Templates" " ForthCodeTemplates" " Alt+J" AddMenuItem drop
  " SearchMenu" # -1 " -" " " " " AddMenuItem drop
  " SearchMenu" # -1 " Jump Class on Cursor" " JumpClassOnCursor" " Ctrl+Alt+F1" AddMenuItem drop
  " SearchMenu" # -1 " Jump Method on Cursor" " JumpMethodOnCursor" " Alt+F1" AddMenuItem drop
  " SearchMenu" # -1 " Jump Class ..." " JumpClassDialog" " Ctrl+Alt+G" AddMenuItem drop
  " SearchMenu" # -1 " Jump Method ..." " JumpMethodDialog" " Shift+Ctrl+Alt+G" AddMenuItem drop
  " SearchMenu" # -1 " Jump Initialization" " JumpInitialization" " Ctrl+Alt+Z" AddMenuItem drop
  " AIDEMenu" # -1 " Register Interface" " RegisterInterface" " Shift+Ctrl+Alt+R" AddMenuItem drop
  " AIDEMenu" # -1 " Try Finally E&x" " TryFinallyEx" " Shift+Alt+Y" AddMenuItem drop

  " ComponentMenu" # 1 " New &General List..." " GeneralList" " Shift+Alt+G" AddMenuItem drop
  " ComponentMenu" # 2 " New &Structured Tree List..." " StructuredTreeList" " Shift+Alt+S" AddMenuItem drop
;

// This word is called every time aide wants to format class

: FormatClass ( SupportedInterfaces AncestorName ClassName -- )
  "   " View . InsertText ( .)                // View.InsertText('  ');
  dup Parser .@ ReadIdent = assert            // Assert(Source.GetIdent = ClasName); // ClassName
  Parser . SkipBlanks ()                      // Source.SkipBlanks;
  true View . BackMargins ( .)                // View.BackMargins(true);
  "  " View . InsertText ( .)                 // View.InsertText(' ');
  " =" View .@ CursorChar = assert            // Assert(View.CursorChar = '='); // =
  View . FeedChar ()                          // View.FeedChar;
  Parser . SkipBlanks ()                      // Source.SkipBlanks;
  true View . BackMargins ( .)                // View.BackMargins(true);
  "  " View . InsertText ( .)                 // View.InsertText(' ');
  Parser .@ ReadIdent                             // Source.GetIdent;  // class
  " packed" = if
    "  " View . InsertText ( .)
    Parser . SkipBlanks ()                    //   Source.SkipBlanks;
    true View . BackMargins ( .)              //   View.BackMargins(true);
    "  " View . InsertText ( .)               //   View.InsertText(' ');
    Parser .@ ReadIdent drop                      //   Source.GetIdent;  // class
  endif
  Parser . SkipBlanks ()                      // Source.SkipBlanks;
  " (" View .@ CursorChar = if                // if View.CursorChar = '(' then begin
    true View . BackMargins ( .)              //   View.BackMargins(true);
    View . FeedChar ()                        //   View.FeedChar;
    Parser . SkipBlanks ()                    //   Source.SkipBlanks;
    true View . BackMargins ( .)              //   View.BackMargins(true);
    over Parser .@ ReadIdent = assert             //   Assert(Source.GetIdent = Ancestor);
    Parser . SkipBlanks ()                    //   Source.SkipBlanks;
    true View . BackMargins ( .)              //   View.BackMargins(true);
    rot " " = if                              //   if SupportedInterfaces = '' then begin
      " )" View .@ CursorChar = assert        //     Assert(View.CursorChar = ')');
      View . FeedChar ()                      //     View.FeedChar;
    else                                      //   end else begin
      # 2 View .! Margin                      //     View.Margin := 2;
      View .@ CursorLine                      //     SaveCursorLine := View.CursorLine;
      while Parser .@ Token dup " ," =        //     while Source.Token in [toSymbol, ','] do begin
        swap # 1 Chr = | loop
        dup View .@ CursorLine =  if          //       if (SaveCursorLine = View.CursorLine)
          View . mCRLF ()                     //         then View.mCRLF
        else                                  //         else begin
          "  " View . InsertText ( .)         //           View.InsertText(' ');
          drop View .@ CursorLine             //           SaveCursorLine := View.CursorLine;
        endif                                 //         end;
        Parser . SkipToken ()                 //       Source.SkipToken;
        Parser . SkipBlanks ()                //       Source.SkipBlanks;
        true View . BackMargins ( .)          //       View.BackMargins(true);
      endloop                                 //     end {while};
      drop View . mCRLF ()                    //     View.mCRLF;
    endif                                     //   end {if};
  else
    drop
  endif                                       // end {if};
  drop drop
;

: FormatScope ( Scope -- )
  "   " View . InsertText ( .)                // View.InsertText('  ');
  dup Parser .@ ReadIdent = assert                // Assert(Source.GetIdent = Ident);
  drop
;

// Formatting return type
: FormatRType
  Parser .@ Token " :" = if
    View . FeedChar ()                        // View.FeedChar;
    Parser . SkipBlanks ()                    // Source.SkipBlanks;
    true View . BackMargins ( .)              // View.BackMargins(true);
    Parser .@ ReadIdent drop                      // Assert(IsValidIdent(Source.GetIdent));
  endif
;

// Formatting of method params
: FormatParams
  Parser .@ Token " (" = if
    // Soon
  endif
  FormatRType
;

: FormatMember ( MemberName -- MemberName )
  "     " View . InsertText ( .)              // View.InsertText('    ');
  dup Parser .@ ReadIdent = assert                // Assert(Source.GetIdent = Ident);
  Parser . SkipBlanks ()                      // Source.SkipBlanks;
  true View . BackMargins ( .)                // View.BackMargins(true);
  "  " View . InsertText ( .)                 // View.InsertText(' ');
  Parser .@ ReadIdent drop                        // Assert(isValidIdent(Source.GetIdent));
  Parser . SkipBlanks ()                      // Source.SkipBlanks;
  true View . BackMargins ( .)                // View.BackMargins(true);

  Parser .@ IdentOnCursor " " = if // property Color default clBtnFace;
  else
    "  " View . InsertText ( .)
  endif
;

: FormatMethod ( MethodPrefix -- )
  FormatMember
  FormatParams           // Left ;
  Parser . SkipBlanks ()                         // Source.SkipBlanks;
  true View . BackMargins ( .)                   // View.BackMargins(true);
  Parser .@ Token " ;" = if
    View . FeedChar ()
    Parser . SkipBlanks ()
  else
    // " ;" View . InsertText ( .) until will not be implemented FormatParams
  endif
  drop
;

: FormatProperty
  " property" FormatMember drop
  Parser .@ Token " [" = if
  endif
  FormatRType
;

Startup
quit

// Temporary suxx code for future block use in begin end
(  View .@ BlockVisible if
    View .@ BlockStartLine View .@ CursorLine =
    View .@ BlockStartCol View .@ CursorCol <= & if true
    else # 0
    endif
  else # 0
  endif )

